/*  ProfileEditP.h

    Private header for  ProfileEdit  widget class.

    Copyright (C) 1997-1999  Richard Gooch

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*

    This include file contains the private class declarations for the
  ProfileEdit widget, a simple profile editing widget for Xt.


    Written by      Richard Gooch   28-FEB-1997

    Last updated by Richard Gooch   21-APR-1999

*/

#ifndef ProfileEditP__H
#define ProfileEditP__H

#include <X11/IntrinsicP.h>
#include <X11/Xaw/FormP.h>
#include <X11/StringDefs.h>

#include <Xkw/ProfileEdit.h>
#include <karma_canvas.h>

typedef struct _vertex
{
    double x;
    double y;
    struct _vertex *left;
    struct _vertex *right;
} *vertex;

typedef struct _ProfileEditPart
{
    /*  Public resources  */
    iarray         array;
    KWorldCanvas   worldCanvas;
    Bool           verbose;
    XtCallbackList callback;
    /*  Private resources  */
    KPixCanvas    pixcanvas;
    KCallbackFunc iarr_destroy_func;
    flag          update_on_drag;
    flag          update_on_drop;
    flag          show_markers;
    int           mode;
    Widget        dressing_control_popup;
    vertex        left_vertex;
    vertex        right_vertex;
    vertex        moving_vertex;
} ProfileEditPart, *ProfileEditPartPtr;

typedef struct _ProfileEditRec
{
    CorePart core;
    CompositePart composite;
    ConstraintPart constraint;
    FormPart form;
    ProfileEditPart profileEdit;
} ProfileEditRec, *ProfileEditPtr;

typedef struct _ProfileEditClassPart
{
    int empty;
} ProfileEditClassPart;

typedef struct _ProfileEditClassRec
{
    CoreClassPart core_class;	
    CompositeClassPart composite_class;
    ConstraintClassPart constraint_class;
    FormClassPart form_class;
    ProfileEditClassPart profileEdit_class;
} ProfileEditClassRec, *ProfileEditClassPtr;

extern ProfileEditClassRec profileEditClassRec;

typedef struct {int empty;} ProfileEditConstraintsPart;

typedef struct _ProfileEditConstraintsRec
{
    FormConstraintsPart	  form;
    ProfileEditConstraintsPart ProfileEdit;
} ProfileEditConstraintsRec, *ProfileEditConstraints;

#endif
