/*  karma_ch_hooks.h

    Header for  ch_  package. This file ONLY contains the hook functions

    Copyright (C) 1997-1999  Richard Gooch

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*

    This include file contains all the definitions for the ch_ package in the
    Karma library.


    Written by      Richard Gooch   14-AUG-1997

    Last updated by Richard Gooch   19-AUG-1999

*/
#ifndef KARMA_CH_HOOKS_H
#define KARMA_CH_HOOKS_H


typedef struct channel_hook_type * KChannelHook;

/*  Function types  */
typedef flag (*KChannelFuncClose) (KChannelHook info);
typedef flag (*KChannelFuncFlush) (KChannelHook info);
typedef unsigned int (*KChannelFuncRead)
     (KChannelHook info, char *buffer,
      CONST char **newbuf, unsigned char alignment, unsigned int length);
typedef unsigned int (*KChannelFuncWrite) (KChannelHook info,
					   CONST char *buffer,
					   unsigned int length);
typedef flag (*KChannelFuncSeek) (KChannelHook info, unsigned long position);
typedef flag (*KChannelFuncGetBytesReadable) (KChannelHook info);
typedef flag (*KChannelFuncSkip) (KChannelHook info, uaddr num_bytes);
typedef int (*KChannelFuncGetDescriptor) (KChannelHook info);


typedef struct  /*  All hook functions must be containted herein  */
{
    KChannelFuncClose             close;
    KChannelFuncFlush             flush;
    KChannelFuncRead              read;
    KChannelFuncWrite             write;
    KChannelFuncSeek              seek;
    KChannelFuncGetBytesReadable  get_bytes_readable;
    KChannelFuncSkip              skip;
    KChannelFuncGetDescriptor     get_descriptor;
} KChannelHookFuncs;

EXTERN_FUNCTION (Channel ch_create_generic,
		 (KChannelHook hook_info, KChannelHookFuncs hooks) );
EXTERN_FUNCTION (KChannelHook ch_get_hook_info, (Channel channel) );
EXTERN_FUNCTION (void ch_call_tap_callbacks, () );
EXTERN_FUNCTION (void ch_change_hooks,
		 (Channel channel, KChannelHookFuncs hooks) );


#endif /*  KARMA_CH_HOOKS_H  */
